/*****************************************************************************
 * $CAMITK_LICENCE_BEGIN$
 *
 * CamiTK - Computer Assisted Medical Intervention ToolKit
 * (c) 2001-2025 Univ. Grenoble Alpes, CNRS, Grenoble INP - UGA, TIMC, 38000 Grenoble, France
 *
 * Visit http://camitk.imag.fr for more information
 *
 * This file is part of CamiTK.
 *
 * CamiTK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * CamiTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with CamiTK.  If not, see <http://www.gnu.org/licenses/>.
 *
 * $CAMITK_LICENCE_END$
 ****************************************************************************/
#ifndef COMPONENTEXTENSIONDESCRIPTIONSTATE_H
#define COMPONENTEXTENSIONDESCRIPTIONSTATE_H

#include "WizardState.h"

class ComponentExtensionDescriptionWidget;
class ComponentExtensionCreationState;

// Dependency from cepcoreschema
// Declaration here to avoid declaration in dependant projects.
namespace cepcoreschema {
class ComponentExtension;
}

/**
 * @ingroup group_sdk_application_wizard
 *
 * @brief
 * State to describe component extension
 *
 */
class ComponentExtensionDescriptionState : public WizardState  {

    Q_OBJECT;

public:
    /**  Constructor */
    ComponentExtensionDescriptionState(QString name, ComponentExtensionDescriptionWidget* widget, WizardMainWindow* mainWindow, ComponentExtensionCreationState* parent);

    /**  Destructor */
    ~ComponentExtensionDescriptionState() override = default;

    void resetDomComponentExtension(cepcoreschema::ComponentExtension* domComponentExtension);

protected:
    void onExit(QEvent* event) override;

private:
    cepcoreschema::ComponentExtension* domComponentExtension;

};
#endif
